#include "ruby.h"
#include "ruby/intern.h"
#include <extractor.h>

static VALUE cExtractor;
static VALUE types;

struct rbExctractor {
    EXTRACTOR_ExtractorList *extractor;
};

/* method definition */
static void
ex_free(struct rbExctractor *extractor)
{
    free(extractor);
}

static VALUE ex_s_allocate(VALUE klass) {
    struct rbExctractor *extractor = ALLOC(struct rbExctractor);
    VALUE obj;
    obj = Data_Wrap_Struct(klass, 0, ex_free, extractor);
    return obj;
}

static VALUE ex_initialize(VALUE self) {
    struct rbExctractor *extractor;
    Data_Get_Struct(self, struct rbExctractor, extractor);
    extractor->extractor = EXTRACTOR_loadDefaultLibraries();
    return self;
}

static VALUE ex_get_keywords(VALUE self, VALUE filename) {
    VALUE hash;
    EXTRACTOR_KeywordList *keywords, *k;
    struct rbExctractor *extractor;
    Check_Type(filename, T_STRING);
    Data_Get_Struct(self, struct rbExctractor, extractor);
    keywords = EXTRACTOR_getKeywords(extractor->extractor, RSTRING_PTR(filename));
    hash = rb_hash_new();
    for(k=keywords; k; k=k->next) {
        VALUE ary, key, value;
        key = rb_hash_aref(types, INT2FIX(k->keywordType));
        value = k->keyword ? rb_str_new2(k->keyword) : Qnil;
        ary = rb_hash_aref(hash, key);
        if (ary==Qnil) {
            ary = rb_ary_new();
        }
        rb_ary_push(ary, value);
        rb_hash_aset(hash, key, ary);
    }
    EXTRACTOR_freeKeywords(keywords);
    return hash;
}

static VALUE ex_get_keywords2(VALUE self, VALUE data) {
    VALUE hash;
    EXTRACTOR_KeywordList *keywords, *k;
    struct rbExctractor *extractor;
    Check_Type(data, T_STRING);
    Data_Get_Struct(self, struct rbExctractor, extractor);
    keywords = EXTRACTOR_getKeywords2(extractor->extractor, RSTRING_PTR(data), RSTRING_LEN(data));
    hash = rb_hash_new();
    for(k=keywords; k; k=k->next) {
        VALUE ary, key, value;
        key = rb_hash_aref(types, INT2FIX(k->keywordType));
        value = k->keyword ? rb_str_new2(k->keyword) : Qnil;
        ary = rb_hash_aref(hash, key);
        if (ary==Qnil) {
            ary = rb_ary_new();
        }
        rb_ary_push(ary, value);
        rb_hash_aset(hash, key, ary);
    }
    EXTRACTOR_freeKeywords(keywords);
    return hash;
}

static VALUE ex_get_extractors(VALUE self) {
    VALUE ary;
    EXTRACTOR_ExtractorList *ex;
    struct rbExctractor *extractor;
    Data_Get_Struct(self, struct rbExctractor, extractor);
    ary = rb_ary_new();
    for(ex=extractor->extractor; ex; ex=ex->next) {
        rb_ary_push(ary, rb_ary_new3(2, rb_str_new2(ex->libname), ex->options ? rb_str_new2(ex->options) : Qnil));
    }
    return ary;
}

static VALUE ex_s_binary_encode(VALUE self, VALUE data) {
    char *ptr;
    ptr = EXTRACTOR_binaryEncode((unsigned char *)RSTRING_PTR(data),RSTRING_LEN(data));
    return ptr ? rb_str_new2(ptr) : Qnil;
}

static VALUE ex_s_binary_decode(VALUE self, VALUE encoded) {
    int ret;
    unsigned char *buf;
    size_t size;
    ret = EXTRACTOR_binaryDecode(RSTRING_PTR(encoded), &buf, &size);
    if (ret) {
        rb_raise(rb_eStandardError, "decode error");
    }
    return rb_str_new((char *)buf, size);
}




void Init_extractor() {
    cExtractor = rb_define_class("Extractor", rb_cObject);
    rb_define_alloc_func(cExtractor, ex_s_allocate);
    rb_define_module_function(cExtractor, "binary_encode", ex_s_binary_encode, 1);
    rb_define_module_function(cExtractor, "binary_decode", ex_s_binary_decode, 1);
    rb_define_method(cExtractor, "initialize", ex_initialize, 0);
    rb_define_method(cExtractor, "get_extractors", ex_get_extractors, 0);
    rb_define_method(cExtractor, "get_keywords", ex_get_keywords, 1);
    rb_define_method(cExtractor, "get_keywords2", ex_get_keywords2, 1);
    rb_define_const(cExtractor, "VERSION", INT2FIX(EXTRACTOR_VERSION));
    types = rb_hash_new();
    rb_define_const(cExtractor, "Types", types);
    rb_hash_aset(types, INT2FIX(EXTRACTOR_UNKNOWN), rb_str_new2("unknown"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_FILENAME), rb_str_new2("filename"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_MIMETYPE), rb_str_new2("mimetype"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_TITLE), rb_str_new2("title"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_AUTHOR), rb_str_new2("author"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_ARTIST), rb_str_new2("artist"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_DESCRIPTION), rb_str_new2("description"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_COMMENT), rb_str_new2("comment"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_DATE), rb_str_new2("date"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PUBLISHER), rb_str_new2("publisher"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_LANGUAGE), rb_str_new2("language"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_ALBUM), rb_str_new2("album"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_GENRE), rb_str_new2("genre"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_LOCATION), rb_str_new2("location"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_VERSIONNUMBER), rb_str_new2("versionnumber"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_ORGANIZATION), rb_str_new2("organization"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_COPYRIGHT), rb_str_new2("copyright"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_SUBJECT), rb_str_new2("subject"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_KEYWORDS), rb_str_new2("keywords"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CONTRIBUTOR), rb_str_new2("contributor"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_RESOURCE_TYPE), rb_str_new2("resource_type"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_FORMAT), rb_str_new2("format"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_RESOURCE_IDENTIFIER), rb_str_new2("resource_identifier"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_SOURCE), rb_str_new2("source"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_RELATION), rb_str_new2("relation"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_COVERAGE), rb_str_new2("coverage"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_SOFTWARE), rb_str_new2("software"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_DISCLAIMER), rb_str_new2("disclaimer"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_WARNING), rb_str_new2("warning"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_TRANSLATED), rb_str_new2("translated"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CREATION_DATE), rb_str_new2("creation_date"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_MODIFICATION_DATE), rb_str_new2("modification_date"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CREATOR), rb_str_new2("creator"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PRODUCER), rb_str_new2("producer"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PAGE_COUNT), rb_str_new2("page_count"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PAGE_ORIENTATION), rb_str_new2("page_orientation"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PAPER_SIZE), rb_str_new2("paper_size"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_USED_FONTS), rb_str_new2("used_fonts"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PAGE_ORDER), rb_str_new2("page_order"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CREATED_FOR), rb_str_new2("created_for"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_MAGNIFICATION), rb_str_new2("magnification"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_RELEASE), rb_str_new2("release"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_GROUP), rb_str_new2("group"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_SIZE), rb_str_new2("size"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_SUMMARY), rb_str_new2("summary"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PACKAGER), rb_str_new2("packager"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_VENDOR), rb_str_new2("vendor"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_LICENSE), rb_str_new2("license"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_DISTRIBUTION), rb_str_new2("distribution"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_BUILDHOST), rb_str_new2("buildhost"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_OS), rb_str_new2("os"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_DEPENDENCY), rb_str_new2("dependency"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_HASH_MD4), rb_str_new2("hash_md4"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_HASH_MD5), rb_str_new2("hash_md5"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_HASH_SHA0), rb_str_new2("hash_sha0"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_HASH_SHA1), rb_str_new2("hash_sha1"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_HASH_RMD160), rb_str_new2("hash_rmd160"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_RESOLUTION), rb_str_new2("resolution"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CATEGORY), rb_str_new2("category"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_BOOKTITLE), rb_str_new2("booktitle"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PRIORITY), rb_str_new2("priority"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CONFLICTS), rb_str_new2("conflicts"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_REPLACES), rb_str_new2("replaces"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PROVIDES), rb_str_new2("provides"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CONDUCTOR), rb_str_new2("conductor"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_INTERPRET), rb_str_new2("interpret"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_OWNER), rb_str_new2("owner"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_LYRICS), rb_str_new2("lyrics"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_MEDIA_TYPE), rb_str_new2("media_type"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CONTACT), rb_str_new2("contact"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_THUMBNAIL_DATA), rb_str_new2("thumbnail_data"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_PUBLICATION_DATE), rb_str_new2("publication_date"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CAMERA_MAKE), rb_str_new2("camera_make"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_CAMERA_MODEL), rb_str_new2("camera_model"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_EXPOSURE), rb_str_new2("exposure"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_APERTURE), rb_str_new2("aperture"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_EXPOSURE_BIAS), rb_str_new2("exposure_bias"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_FLASH), rb_str_new2("flash"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_FLASH_BIAS), rb_str_new2("flash_bias"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_FOCAL_LENGTH), rb_str_new2("focal_length"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_FOCAL_LENGTH_35MM), rb_str_new2("focal_length_35mm"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_ISO_SPEED), rb_str_new2("iso_speed"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_EXPOSURE_MODE), rb_str_new2("exposure_mode"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_METERING_MODE), rb_str_new2("metering_mode"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_MACRO_MODE), rb_str_new2("macro_mode"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_IMAGE_QUALITY), rb_str_new2("image_quality"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_WHITE_BALANCE), rb_str_new2("white_balance"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_ORIENTATION), rb_str_new2("orientation"));
    rb_hash_aset(types, INT2FIX(EXTRACTOR_TEMPLATE), rb_str_new2("template"));
}
